package vtm_engine;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

public class VTM {
	
	private static char GRAPHICAL;
	private static char TESTING;
	private final static String TITLE = "Virtual Turing Machine: BETA 0.2.0";

	private static Scanner in, instructionTableInput, tapeInput;
	private static Tape tape;
	private static RWHead rwHead;
	private static StateRegister stateRegister;
	private static InstructionTable instructionTable;
	private static VTM_GUI_INPUT G_VTM_IN;
	private static VTM_GUI_OUTPUT G_VTM_OUT;
	
	/**
	 * @param args
	 * @throws FileNotFoundException 
	 */
	public static void main(String[] args) throws FileNotFoundException{
		System.out.println(TITLE);
		
		/*
		 * The current version of the software accepts input from the command
		 * line in the form of files designated to be Instruction Tables and
		 * Tapes. The program can read the files in and then parse them, and
		 * run the TM.
		 */
		
		GRAPHICAL = JOptionPane.showInputDialog("Use GUI? Y/N").charAt(0);
		//TESTING = JOptionPane.showInputDialog("Testing? Y/N").charAt(0);
		TESTING = 'N';
		
		/*
		 * If we are NOT using the GUI to run the program, but rather the command line
		 */
		if (GRAPHICAL == 'N') {
			in = new Scanner(System.in);
			
			if (TESTING == 'N') {
				System.out.println("Please Enter the Name of the Instruction Table File");
				instructionTableInput = new Scanner (new File( in.next() ));
				System.out.println("Please Enter the Name of the Tape File");
				tapeInput = new Scanner (new File( in.next() ));
			} else {
				System.out.println("TESTING USING 'res/BusyBeaverInstructionTable.txt' & 'res/BusyBeaverTape.txt'");
				instructionTableInput = new Scanner (new File( "res/BusyBeaverInstructionTable.txt" ));
				tapeInput = new Scanner (new File( "res/BusyBeaverTape.txt" ));
			}
		
			tape = new Tape();
			rwHead = new RWHead();
			stateRegister = new StateRegister();
			instructionTable = new InstructionTable();
		
			readInstructionTableFile();
			instructionTable.printInstructionTable(instructionTable.getInstructionCount());
			readTapeFile();
			tape.printTape(tape.getLength());
		
			System.out.println("Please Enter The Starting State: ");
			char startingState = in.next().charAt(0);
			System.out.println("Please Enter The Initial State of The Read Register: ");
			int startingRead = in.nextInt();
			System.out.println("Please Enter The Name Of The TM: ");
			String name = in.next();
		
			runTM(startingState, startingRead, name);
		}
		
		
		/*
		 * If we are using the GUI to run the program
		 */
		else {
			G_VTM_IN = new VTM_GUI_INPUT(TITLE);
			G_VTM_IN.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			G_VTM_IN.setSize(600, 250);
			G_VTM_IN.setResizable(false);
			G_VTM_IN.setLocationRelativeTo(null);
			G_VTM_IN.setVisible(true);
			
			tape = new Tape();
			rwHead = new RWHead();
			stateRegister = new StateRegister();
			instructionTable = new InstructionTable();
			
			int counter = 0;
			while (!G_VTM_IN.isRunning()) { 
				//Wait for the G_VTM Run button to be pressed
				System.out.print("");
				counter++;
			}
			
			if (TESTING != 'N') {
				System.out.println("EXITED WHILE LOOP AFTER :: " + counter + " ITERATIONS OF WHILE LOOP");
			
				System.out.println(G_VTM_IN.getInstructionTable_String());
				System.out.println(G_VTM_IN.getTape_String());
				System.out.println(G_VTM_IN.getStartingState_String().charAt(0));
				//System.out.println(Integer.parseInt(G_VTM_IN.getRWHead_String()));
				System.out.println(G_VTM_IN.getProgramName_String());
			}
			
			instructionTableInput = new Scanner (new File( G_VTM_IN.getInstructionTable_String() ));
			readInstructionTableFile();
			instructionTable.printInstructionTable(instructionTable.getInstructionCount());
			tapeInput = new Scanner (new File( G_VTM_IN.getTape_String() ));
			readTapeFile();
			tape.printTape(tape.getLength());
			char startingState = G_VTM_IN.getStartingState_String().charAt(0);
			//int startingRead = Integer.parseInt(G_VTM_IN.getRWHead_String());
			String name = G_VTM_IN.getProgramName_String();
			
			G_VTM_OUT = new VTM_GUI_OUTPUT(name);
			G_VTM_OUT.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			G_VTM_OUT.setSize(600, 250);
			G_VTM_OUT.setResizable(false);
			G_VTM_OUT.setLocationRelativeTo(null);
			G_VTM_OUT.setVisible(true);
		
			runGraphicalTM(startingState, 0, name);
		}
	}//end of main(String[] args)
	
	/**
	 * 
	 */
	public static void readInstructionTableFile() {
		//System.out.println("Entering 'readInstructionTableFile()'");
		while (instructionTableInput.hasNext()) {
			String line = instructionTableInput.nextLine();
			instructionTable.addInstruction(new Instruction(line.charAt(0), line.charAt(2), line.charAt(4),
					line.charAt(6), line.charAt(8)));
		}
	}//end of readInstructionTableFile()
	
	/**
	 * 
	 */
	public static void readTapeFile() {
		//System.out.println("Entering 'readTapeFile()'");
		while (tapeInput.hasNext()) {
			String line = tapeInput.nextLine();
			for (int i = 0; i < line.length(); i++) {
				if (line.charAt(i) != ' ' ){
					tape.addCharacter(line.charAt(i));
				}
			}
		}
	}//end of readTapeFile()
	
	/**
	 * 
	 */
	public static void runTM(char stateState, int startIndex, String name) {
		System.out.println("Running TM " + name);
		stateRegister.setCurrentState(stateState);
		rwHead.setIndex(startIndex);
		
		boolean running = true;
		while (running) {
			char currentRead = tape.getCharacter(rwHead.getIndex());
			System.out.println("Current Character Read: " + currentRead);
			System.out.println("Current State: " + stateRegister.getCurrentState());
			tape.printTape(tape.getLength());
			boolean looking = true;
			
			for (int i = 0; i < instructionTable.getInstructionCount(); i++) {
				//check for the correct instruction
				if (stateRegister.getCurrentState() == instructionTable.getInstruction(i).getState() 
						&& instructionTable.getInstruction(i).getRead() == currentRead && looking){
					System.out.print("Instruction Found! ");
					instructionTable.getInstruction(i).printInstruction();
					
					tape.setCharacter(rwHead.getIndex(), instructionTable.getInstruction(i).getWrite());
					System.out.println("Writing: " + instructionTable.getInstruction(i).getWrite() 
							+ " Location: " + rwHead.getIndex());
					stateRegister.setCurrentState(instructionTable.getInstruction(i).getNextState());
					
					if(instructionTable.getInstruction(i).isMoveRight()) {
						rwHead.setIndex(rwHead.getIndex() + 1);
						System.out.println("RW Head Moving Right");
					} 
					else {
						rwHead.setIndex(rwHead.getIndex() - 1);
						System.out.println("RW Head Moving Left");
					}
					looking = false;
				}
				if (stateRegister.getCurrentState() == instructionTable.getInstruction(i).getState() 
						&& instructionTable.getInstruction(i).getNextState() == '#' && looking) {
					running = false;
					looking = false;
					tape.setCharacter(rwHead.getIndex(), instructionTable.getInstruction(i).getWrite());
					System.out.println("Writing: " + instructionTable.getInstruction(i).getWrite() 
							+ " Location: " + rwHead.getIndex());
					tape.printTape(tape.getLength());
					System.out.println("HALT INSTRUCTION FOUND");
				}
			}//end of for() loop
			try {
				Thread.sleep(300);
			} catch (InterruptedException e) { e.printStackTrace(); }
		}//end of while(running)
	}//end of runTM()
	
	public static void runGraphicalTM(char stateState, int startIndex, String name) {
		//System.out.println("Entering Graphical TM " + name);
		stateRegister.setCurrentState(stateState);
		rwHead.setIndex(startIndex);
		
		boolean running = true;
		while (running) {
			char currentRead = tape.getCharacter(rwHead.getIndex());
			G_VTM_OUT.setRWHead(Character.toString(currentRead));
			G_VTM_OUT.setCurrentState(Character.toString(stateRegister.getCurrentState()));
			G_VTM_OUT.setTape(tape.toString());
			G_VTM_OUT.setInstructionTable(instructionTable.toString());
			boolean looking = true;
			
			while (!G_VTM_OUT.isStepButtonPressed()) { 
				//Wait for the G_VTM Run button to be pressed
				System.out.print("");
				G_VTM_OUT.drawRect(rwHead.getIndex());
			}
			
			if (G_VTM_OUT.isRunButtonPressed()) {
				//if the VTM is on running mode, 
				//have to display rwHead rectangle
				int counter = 0;
				while (counter < 10000) {
					G_VTM_OUT.drawRect(rwHead.getIndex());
					counter++;
				}
			}
			
			for (int i = 0; i < instructionTable.getInstructionCount(); i++) {
				//check for the correct instruction
				if (stateRegister.getCurrentState() == instructionTable.getInstruction(i).getState() 
						&& instructionTable.getInstruction(i).getRead() == currentRead && looking){
					
					G_VTM_OUT.setCurrentInstruction(instructionTable.getInstruction(i).toString());
					
					tape.setCharacter(rwHead.getIndex(), instructionTable.getInstruction(i).getWrite());
					//System.out.println("Writing: " + instructionTable.getInstruction(i).getWrite() 
					//		+ " Location: " + rwHead.getIndex());
					stateRegister.setCurrentState(instructionTable.getInstruction(i).getNextState());
					
					if (instructionTable.getInstruction(i).isMoveRight()) {
						rwHead.setIndex(rwHead.getIndex() + 1);
						//System.out.println("RW Head Moving Right");
					} 
					else {
						rwHead.setIndex(rwHead.getIndex() - 1);
						//System.out.println("RW Head Moving Left");
					}
					looking = false;
				}
				if (stateRegister.getCurrentState() == instructionTable.getInstruction(i).getState() 
						&& instructionTable.getInstruction(i).getNextState() == '#' && looking) {
					running = false;
					looking = false;
					tape.setCharacter(rwHead.getIndex(), instructionTable.getInstruction(i).getWrite());
					G_VTM_OUT.setTape(tape.toString());
					G_VTM_OUT.drawRect(rwHead.getIndex());
				}
			}//end of for() loop
			try {
				Thread.sleep(300);
			} catch (InterruptedException e) { e.printStackTrace(); }
			if (!G_VTM_OUT.isRunButtonPressed()) {
				G_VTM_OUT.setStepButtonPressed(false);
			}
		}//end of while(running)
	}

	
	
	
	
	/**
	 * 
	 */
	public static void exampleProg() {
		//public Instruction(char state, char read, char write, char nextState, boolean moveRight)
		System.out.println("RUNNING EXAMPLE PROGRAM.");
		
		System.out.println("BUSY BEAVER TM - TEST");
		instructionTable.addInstruction(new Instruction('A', '0', '1', 'B', true));
		instructionTable.addInstruction(new Instruction('A', '1', '0', 'C', false));
		instructionTable.addInstruction(new Instruction('B', '0', '1', 'A', true));
		instructionTable.addInstruction(new Instruction('B', '1', '0', 'C', false));
		instructionTable.addInstruction(new Instruction('C', '0', '$', '#', false));
		instructionTable.addInstruction(new Instruction('C', '1', '$', '#', false));
		
		for (int i = 0; i < 16; i++) {
			if (i < 15) {
				tape.setCharacter(i, '0');
				tape.setLength(tape.getLength() + 1);
			} else {
				tape.setCharacter(i, '1');
				tape.setLength(tape.getLength() + 1);
			}
		}
		
		instructionTable.printInstructionTable(instructionTable.getInstructionCount());
		tape.printTape(tape.getLength());
		
		runTM('A', 0, "Busy Beaver");
	}//end of exampleProg()
	
}//end of public class VTM
