package vtm_engine;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class VTM_GUI_OUTPUT extends JFrame {
	private static final long serialVersionUID = 1L;
	
	private static final int REGISTER_TEXT_FIELD_SIZE = 3;
	private static final int CURRENT_INSTRUCTION_TEXT_FIELD_SIZE = 10;
	private static final int TAPE_TEXT_AREA_WIDTH = 30;
	private static final int TAPE_TEXT_AREA_HEIGHT = 3;
	private static final int RECT_HEIGHT = 13;
	private static final int RECT_WIDTH = 9;
	private static final int RECT_Y = 161;
	private static final int RECT_X = 20;
	private static final int CHAR_SIZE = 12;
	private static final int ROW_SIZE = 16;
	
	private static int charOffset;
	
	private JPanel registers_Panel;
	private JPanel instructionTable_Panel;
	private JPanel tape_Panel;
	
	private JLabel rwHead_Label;
	private JLabel currentState_Label;
	private JLabel currentInstruction_Label;
	
	private JTextField rwHead_TextField;
	private JTextField currentState_TextField;
	private JTextField currentInstruction_TextField;
	
	private JTextArea tape_TextArea;
	private JTextArea instructionTable_TextArea;
	
	private JButton run_Button;
	private JButton step_Button;
	
	private boolean runButtonPressed;
	private boolean stepButtonPressed;
		
	private VTM_ActionListener buttonListener;

	public VTM_GUI_OUTPUT(String title) {
		super(title);
		runButtonPressed = false;
		stepButtonPressed = false;
		setLayout(new FlowLayout());
		initComponents();
	}
	
	public void initComponents() {
		registers_Panel = new JPanel();
		registers_Panel.setLayout(new GridLayout(3, 2));
		
		instructionTable_Panel = new JPanel();
		instructionTable_Panel.setLayout(new FlowLayout());
		
		tape_Panel = new JPanel();
		tape_Panel.setLayout(new FlowLayout());
		tape_Panel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		rwHead_Label = new JLabel("RW Head Value: ");
		registers_Panel.add(rwHead_Label);
		rwHead_TextField = new JTextField(REGISTER_TEXT_FIELD_SIZE);
		registers_Panel.add(rwHead_TextField);
		
		currentState_Label = new JLabel("Current State: ");
		registers_Panel.add(currentState_Label);
		currentState_TextField = new JTextField(REGISTER_TEXT_FIELD_SIZE);
		registers_Panel.add(currentState_TextField);
		
		currentInstruction_Label = new JLabel("Current Instruction: ");
		registers_Panel.add(currentInstruction_Label);
		currentInstruction_TextField = new JTextField(CURRENT_INSTRUCTION_TEXT_FIELD_SIZE);
		registers_Panel.add(currentInstruction_TextField);
		
		instructionTable_TextArea = new JTextArea(7, 12);
		instructionTable_TextArea.setPreferredSize(new Dimension(7, 12));
		instructionTable_TextArea.setLineWrap(true);
		instructionTable_TextArea.setEditable(false);
		instructionTable_Panel.add(instructionTable_TextArea);
		
		tape_TextArea = new JTextArea(TAPE_TEXT_AREA_HEIGHT, TAPE_TEXT_AREA_WIDTH);
		tape_TextArea.setPreferredSize(new Dimension(TAPE_TEXT_AREA_HEIGHT, TAPE_TEXT_AREA_WIDTH));
		tape_TextArea.setLineWrap(true);
		//tape_TextArea.setAutoscrolls(true);
		tape_TextArea.setEditable(false);
		tape_Panel.add(tape_TextArea);
		
		charOffset = 0;
		
		buttonListener = new VTM_ActionListener();
		
		run_Button = new JButton("Run TM");
		run_Button.addActionListener(buttonListener);
		tape_Panel.add(run_Button);
		
		step_Button = new JButton("Step TM");
		step_Button.addActionListener(buttonListener);
		tape_Panel.add(step_Button);
		
		add(registers_Panel);
		add(instructionTable_Panel);
		add(tape_Panel);
	}
	
	public void drawRect(int index) {
		int row_index = 0;
		charOffset = 0;
		if (index > 29 && index <= 59) {
			row_index = 1;
			charOffset = 4;
		}
		else if (index > 59 && index <= 89) {
			row_index = 2;
			charOffset = 4;
		}
		else if (index > 89 && index <= 119) {
			row_index = 3;
			charOffset = 4;
		}
		index = index % TAPE_TEXT_AREA_WIDTH;
		Graphics g = getGraphics();
		g.setColor(Color.RED);
		g.drawRect(RECT_X + (CHAR_SIZE * index) + charOffset, RECT_Y + (ROW_SIZE * row_index), RECT_WIDTH, RECT_HEIGHT);
	}
	
	public void setRWHead(String s) {
		rwHead_TextField.setText(s);
	}
	
	public void setCurrentState(String s) {
		currentState_TextField.setText(s);
	}
	
	public void setCurrentInstruction(String s) {
		currentInstruction_TextField.setText(s);
	}
	
	public void setInstructionTable(String s) {
		instructionTable_TextArea.setText(s);
	}
	
	public void setTape(String s) {
		tape_TextArea.setText(s);
		tape_TextArea.setAlignmentX(LEFT_ALIGNMENT);
	}
	
	public boolean isRunButtonPressed() {
		return runButtonPressed;
	}
	
	public void setStepButtonPressed(boolean b) {
		stepButtonPressed = b;
	}
	
	public boolean isStepButtonPressed() {
		return stepButtonPressed;
	}
	
	private class VTM_ActionListener implements ActionListener {
		
		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == run_Button) {
				//System.out.println("Run Button Pressed");
				runButtonPressed = true;
				stepButtonPressed = true;
			}
			if (event.getSource() == step_Button) {
				//System.out.println("Step Button Pressed");
				stepButtonPressed = true;
			}
		}
	}
}
